#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright 2022 Shanghai AI Lab. Licensed under MIT License.
#

"""Dataset related classes."""

from opendatalab.dataset.dataset import Dataset

__all__ = [
    "Dataset",
]
